<?php

namespace EventPortal\Concerns;

use EventPortal\Contracts\HasFields as HasFieldsContract;
use EventPortal\Domain\Component\Value;
use EventPortal\Domain\Field\Actions\InitialiseFields;
use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Response\Actions\SaveToFields;
use Illuminate\Database\Eloquent\Relations\MorphMany;

trait HasFields
{
    public function sections(): MorphMany
    {
        return $this->fields()->onlySections();
    }

    public function fields(): MorphMany
    {
        return $this->morphMany(Field::class, 'model');
    }

    public function initialiseFields(array $content): HasFieldsContract
    {
        app(InitialiseFields::class)->execute($content, $this);

        return $this;
    }

    public function saveToFields(array $data): HasFieldsContract
    {
        app(SaveToFields::class)->execute($this, $data);

        return $this->fresh();
    }

    // phpcs:disable
    /**
     * Unable to type hint to avoid breaking compatibility with base __get
     *
     * @param string $key The property you're trying to access
     * @return mixed
     */
    public function __get($key)
    {
        // phpcs:enable
        $data = parent::__get($key);

        if ($data !== null) {
            return $data;
        }

        $field = Field::forModel($this)->firstWhere('name', $key);

        if ($field === null || $field->response === null) {
            return null;
        }

        return Value::get($field->response->value);
    }
}
