<?php

namespace EventPortal\Domain\Util;

use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Form\Form;
use Illuminate\Support\Arr;

class ValidationRules
{
    public static function forForm(Form $form, string $action): array
    {
        if ($form->hasInstanceOfModel()) {
            return $form
                ->getModel()
                ->fields
                ->reject(function (Field $field) {
                    return $field->type->isSection();
                })
                ->mapWithKeys(function (Field $field) use ($action): array {
                    return [$field->name => CombinedValidation::get($field, $action)];
                })
                ->filter()
                ->toArray();
        }

        return collect($form->content)
            ->flatMap(function ($section) {
                return Arr::get($section, 'children');
            })
            ->mapWithKeys(function ($field) use ($action) {
                return [
                    Arr::get($field, 'name') => CombinedValidation::get($field, $action),
                ];
            })
            ->filter()
            ->toArray();
    }
}
