<?php

namespace EventPortal\Tests\Feature\Admin\Event;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Response\Models\Response;
use EventPortal\Domain\Users\Models\User;
use EventPortal\Tests\Support\Traits\InteractsWithTeams;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Spatie\Snapshots\MatchesSnapshots;

class EventShowTest extends TestCase
{
    use DatabaseMigrations, InteractsWithTeams, MatchesSnapshots;

    /** @test */
    public function show()
    {
        $user = factory(User::class)->create();
        $event = factory(Event::class)->create();

        $this->addToSameTeam([$user, $event]);

        $section = factory(Field::class)
            ->states('Section')
            ->create([
                'label' => 'Key details',
                'model_id' => $event->id,
                'model_type' => get_class($event),
            ]);

        $fieldA = factory(Field::class)
            ->states('TextInput')
            ->create([
                'label' => 'Event name',
                'name' => 'name',
                'model_id' => $event->id,
                'model_type' => get_class($event),
                'section_id' => $section->id,
            ]);

        $fieldB = factory(Field::class)
            ->states('Select')
            ->create([
                'label' => 'Event type',
                'name' => 'event_type',
                'model_id' => $event->id,
                'model_type' => get_class($event),
                'section_id' => $section->id,
                'data' => [
                    'options' => [
                        ['label' => 'Seminar', 'value' => 'seminar'],
                        ['label' => 'Single', 'value' => 'single'],
                    ],
                ],
            ]);

        factory(Response::class)
            ->create([
                'field_id' => $fieldA->id,
                'value' => 'Hello World!',
            ]);

        factory(Response::class)
            ->create([
                'field_id' => $fieldB->id,
                'value' => ['label' => 'Single', 'value' => 'single'],
            ]);

        $this->assertMatchesJsonSnapshot(
            $this->actingAs($user)
                ->get(route('admin.event.show', $event))->props()
        );
    }
}
