<?php

namespace EventPortal\Tests\Feature\Admin\Team;

use EventPortal\Tests\TestCase;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Domain\Users\Models\User;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Response\Models\Response;
use Illuminate\Foundation\Testing\DatabaseMigrations;

class TeamTest extends TestCase
{
    use DatabaseMigrations;

    /** @test */
    public function a_user_cannot_view_events_from_other_teams()
    {
        list($team1, $team2) = factory(Team::class, 2)->create();
        list($team1Event, $team2Event) = factory(Event::class, 2)->create();
        list($team1User, $team2User) = factory(User::class, 2)->create();

        $this->setUpEvent($team1Event);
        $this->setUpEvent($team2Event);

        $team1->events()->sync($team1Event->id);
        $team1->users()->sync($team1User->id);

        $team2->events()->sync($team2Event->id);
        $team2->users()->sync($team2User->id);

        $this->actingAs($team1User)
            ->get(route('admin.event.show', $team1Event))
            ->assertOk();

        $this->get(route('admin.event.show', $team2Event))
            ->assertUnauthorized();

        $this->actingAs($team2User)
            ->get(route('admin.event.show', $team2Event))
            ->assertOk();

        $this->get(route('admin.event.show', $team1Event))
            ->assertUnauthorized();
    }

    private function setUpEvent($event)
    {
        $section = factory(Field::class)
            ->states('Section')
            ->create([
                'label' => 'Key details',
                'model_id' => $event->id,
                'model_type' => get_class($event),
            ]);

        $fieldA = factory(Field::class)
            ->states('TextInput')
            ->create([
                'label' => 'Event name',
                'name' => 'name',
                'model_id' => $event->id,
                'model_type' => get_class($event),
                'section_id' => $section->id,
            ]);

        $fieldB = factory(Field::class)
            ->states('Select')
            ->create([
                'label' => 'Event type',
                'name' => 'event_type',
                'model_id' => $event->id,
                'model_type' => get_class($event),
                'section_id' => $section->id,
                'data' => [
                    'options' => [
                        ['label' => 'Seminar', 'value' => 'seminar'],
                        ['label' => 'Single', 'value' => 'single'],
                    ],
                ],
            ]);

        factory(Response::class)
            ->create([
                'field_id' => $fieldA->id,
                'value' => 'Hello World!',
            ]);

        factory(Response::class)
            ->create([
                'field_id' => $fieldB->id,
                'value' => ['label' => 'Single', 'value' => 'single'],
            ]);
    }
}
