<?php

namespace EventPortal\Tests\Unit\Component;

use EventPortal\Domain\Component\Dropdown\MultiSelect;
use EventPortal\Domain\Component\Dropdown\Select;
use EventPortal\Domain\Component\Facades\Input;
use EventPortal\Domain\Component\Input\PasswordInput;
use EventPortal\Domain\Component\Input\TextArea;
use EventPortal\Domain\Component\Input\TextInput;
use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Component\Selection\Checkbox;
use EventPortal\Domain\Component\Selection\RadioButtons;
use EventPortal\Domain\Component\Selection\SlideToggle;
use EventPortal\Domain\Field\Enums\FieldType;
use EventPortal\Tests\TestCase;

class InputFromArrayTest extends TestCase
{
    /** @test */
    public function can_make_text_input()
    {
        $field = [
            'component' => FieldType::TextInput()->getValue(),
            'name' => 'name',
            'label' => 'Event name',
            'classes' => ['col-6'],
            'validation' => [
                'required',
            ],
            'placeholder' => 'E.g. Congress meeting',
        ];

        $component = Input::fromArray($field);

        $this->assertInstanceOf(TextInput::class, $component);
    }

    /** @test */
    public function can_make_select()
    {
        $field = [
            'component' => FieldType::Select()->getValue(),
            'name' => 'type',
            'label' => 'Event Type',
            'classes' => ['col-6'],
            'placeholder' => 'Select a event component',
            'options' => [
                Option::make('Single'),
                Option::make('Seminar'),
            ],
            'validation' => [
                'required',
            ],
        ];

        $component = Input::fromArray($field);

        $this->assertInstanceOf(Select::class, $component);
    }

    /** @test */
    public function can_make_radio_buttons()
    {
        $field = [
            'component' => FieldType::RadioButtons()->getValue(),
            'name' => 'radio_buttons',
            'label' => 'RadioButtons Buttons',
            'classes' => ['col-6'],
            'placeholder' => 'Select choose',
            'options' => [
                Option::make('Yes'),
                Option::make('No'),
            ],
        ];

        $component = Input::fromArray($field);

        $this->assertInstanceOf(RadioButtons::class, $component);
    }

    /** @test */
    public function can_make_password_input()
    {
        $field = [
            'component' => FieldType::PasswordInput()->getValue(),
            'label' => 'Event password',
            'name' => 'password',
            'classes' => ['col-6'],
            'clearable' => true,
        ];

        $component = Input::fromArray($field);

        $this->assertInstanceOf(PasswordInput::class, $component);
    }

    /** @test */
    public function can_make_text_area()
    {
        $field = [
            'component' => FieldType::TextArea()->getValue(),
            'name' => 'description',
            'label' => 'Event description',
            'classes' => ['col-6'],
        ];

        $component = Input::fromArray($field);

        $this->assertInstanceOf(TextArea::class, $component);
    }

    /** @test */
    public function can_make_multi_select()
    {
        $field = [
            'component' => FieldType::MultiSelect()->getValue(),
            'name' => 'multi_select',
            'label' => 'Multiselect',
            'classes' => ['col-6'],
            'placeholder' => 'Select something',
            'options' => [
                Option::make('Toby')->value(1),
                Option::make('Andy')->value(2),
                Option::make('Fadhil')->value(3),
            ],
        ];

        $component = Input::fromArray($field);

        $this->assertInstanceOf(MultiSelect::class, $component);
    }

    /** @test */
    public function can_make_checkbox()
    {
        $field = [
            'component' => FieldType::Checkbox()->getValue(),
            'name' => 'check_do_something',
            'label' => 'Check this to do something',
            'classes' => ['col-6'],
        ];

        $component = Input::fromArray($field);

        $this->assertInstanceOf(Checkbox::class, $component);
    }

    /** @test */
    public function can_make_slide_toggle()
    {
        $field = [
            'component' => FieldType::SlideToggle()->getValue(),
            'name' => 'slide_toggle',
            'label' => 'Slidetoggle',
            'classes' => ['col-6'],
        ];

        $component = Input::fromArray($field);

        $this->assertInstanceOf(SlideToggle::class, $component);
    }

    /** @test **/
    public function made_with_label()
    {
        $field = [
            'component' => FieldType::TextInput()->getValue(),
            'name' => 'name',
            'label' => 'Event name',
        ];

        $component = Input::fromArray($field);

        $this->assertEquals('Event name', $component->label);
    }

    /** @test **/
    public function made_with_placeholder()
    {
        $field = [
            'component' => FieldType::TextInput()->getValue(),
            'name' => 'name',
            'label' => 'Event name',
            'placeholder' => 'Hello World'
        ];

        $component = Input::fromArray($field);

        $this->assertEquals('Hello World', $component->placeholder);
    }

    /** @test **/
    public function made_with_clearable()
    {
        $field = [
            'component' => FieldType::TextInput()->getValue(),
            'name' => 'name',
            'label' => 'Event name',
            'clearable' => true,
        ];

        $component = Input::fromArray($field);

        $this->assertTrue($component->clearable);
    }

    /** @test **/
    public function made_with_classes()
    {
        $field = [
            'component' => FieldType::TextInput()->getValue(),
            'name' => 'name',
            'label' => 'Event name',
            'classes' => ['col-6', 'no-margin'],
        ];

        $component = Input::fromArray($field);

        $this->assertEquals('col-6', $component->classes[0]);
        $this->assertEquals('no-margin', $component->classes[1]);
    }

    /** @test **/
    public function made_with_validation()
    {
        $field = [
            'component' => FieldType::TextInput()->getValue(),
            'name' => 'name',
            'label' => 'Event name',
            'validation' => ['required', 'email']
        ];

        $component = Input::fromArray($field);

        $this->assertEquals('required', $component->validation[0]);
        $this->assertEquals('email', $component->validation[1]);
        $this->assertTrue($component->required);
    }

    /** @test **/
    public function made_with_create_validation()
    {
        $field = [
            'component' => FieldType::TextInput()->getValue(),
            'name' => 'name',
            'label' => 'Event name',
            'createValidation' => ['required', 'email']
        ];

        $component = Input::fromArray($field);

        $this->assertEquals('required', $component->createValidation[0]);
        $this->assertEquals('email', $component->createValidation[1]);
    }

    /** @test **/
    public function made_with_update_validation()
    {
        $field = [
            'component' => FieldType::TextInput()->getValue(),
            'name' => 'name',
            'label' => 'Event name',
            'updateValidation' => ['required', 'email']
        ];

        $component = Input::fromArray($field);

        $this->assertEquals('required', $component->updateValidation[0]);
        $this->assertEquals('email', $component->updateValidation[1]);
    }

    /** @test **/
    public function made_with_pipe_validation()
    {
        $field = [
            'component' => FieldType::TextInput()->getValue(),
            'name' => 'name',
            'label' => 'Event name',
            'validation' => 'required|email',
            'createValidation' => 'min:5|max:10',
            'updateValidation' => 'boolean|date|array',
        ];

        $component = Input::fromArray($field);

        $this->assertEquals('required', $component->validation[0]);
        $this->assertEquals('email', $component->validation[1]);

        $this->assertEquals('min:5', $component->createValidation[0]);
        $this->assertEquals('max:10', $component->createValidation[1]);

        $this->assertEquals('boolean', $component->updateValidation[0]);
        $this->assertEquals('date', $component->updateValidation[1]);
        $this->assertEquals('array', $component->updateValidation[2]);
    }

    /** @test **/
    public function made_with_options()
    {
        $field = [
            'component' => FieldType::Select()->getValue(),
            'name' => 'options',
            'label' => 'Event options',
            'options' => [
                Option::make('Yes'),
                Option::make('No')
            ],
        ];

        $component = Input::fromArray($field);

        $expected = [
            ['label' => 'Yes', 'value' => 'yes'],
            ['label' => 'No', 'value' => 'no'],
        ];

        $this->assertEquals($expected, $component->options);
    }

    /** @test */
    public function is_not_made_if_there_are_no_options()
    {
        $field = [
            'component' => FieldType::Select()->getValue(),
            'name' => 'name',
            'label' => 'Event name',
        ];

        $component = Input::fromArray($field);

        $this->assertEquals([], $component->options);
    }

    /** @test */
    public function can_set_default_value_for_text()
    {
        $field = [
            'component' => FieldType::TextInput()->getValue(),
            'name' => 'name',
            'label' => 'Event name',
            'defaultValue' => 'Hello World!',
        ];

        $component = Input::fromArray($field);

        $this->assertEquals('Hello World!', $component->value);
    }

    /** @test */
    public function can_set_default_value_for_select()
    {
        $field = [
            'component' => FieldType::Select()->getValue(),
            'name' => 'status',
            'label' => 'Event status',
            'options' => [
                Option::make('Draft'),
                Option::make('Published'),
            ],
            'defaultValue' => Option::make('Draft'),
        ];

        $component = Input::fromArray($field);

        $this->assertEquals(Option::make('Draft'), $component->value);
    }
}
