<?php

namespace EventPortal\Tests\Unit\Event\Actions;

use Carbon\Carbon;
use EventPortal\Domain\Event\Actions\ArchiveEvent;
use EventPortal\Domain\Event\Actions\UnArchiveEvent;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Response\Models\Response;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class ArchiveEventTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_archive_event(): void
    {
        Carbon::setTestNow(Carbon::parse('01-01-2020'));
        $event = factory(Event::class)->create();
        $section = factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'Section',
            'label' => 'Status',
        ]);
        $field = factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'Select',
            'label' => 'Event Status',
            'name' => 'status',
            'section_id' => $section->id,
        ]);
        $response = factory(Response::class)->create([
            'field_id' => $field->id,
        ]);

        Carbon::setTestNow(Carbon::parse('05-01-2020'));

        (new ArchiveEvent)->execute($event);

        $this->assertNotNull($event->fresh());
        $this->assertNotNull($field->fresh());
        $this->assertEquals(['label' => 'Archived', 'value' => 'archived'], $response->fresh()->value);
    }

    /** @test */
    public function can_un_archive_event(): void
    {
        Carbon::setTestNow(Carbon::parse('01-01-2020'));
        $event = factory(Event::class)->create();
        $section = factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'Section',
            'label' => 'Status',
        ]);
        $field = factory(Field::class)->create([
            'model_id' => $event->id,
            'model_type' => Event::class,
            'type' => 'Select',
            'label' => 'Event Status',
            'name' => 'status',
            'section_id' => $section->id,
        ]);

        $response = factory(Response::class)->create([
            'field_id' => $field->id,
        ]);

        Carbon::setTestNow(Carbon::parse('05-01-2020'));

        (new UnArchiveEvent)->execute($event);

        $this->assertNotNull($event->fresh());
        $this->assertNotNull($field->fresh());
        $this->assertEquals(['label' => 'Draft', 'value' => 'draft'], $response->fresh()->value);
    }
}
