<?php

namespace EventPortal\Tests\Unit\Event\Actions;

use EventPortal\Domain\Event\Actions\DeleteEvent;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Response\Models\Response;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class DeleteEventTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_delete_event()
    {
        $eventA = factory(Event::class)->create();
        $fieldA = factory(Field::class)->create([
            'model_id' => $eventA->id,
            'model_type' => Event::class,
        ]);
        $responseA = factory(Response::class)->create([
            'field_id' => $fieldA->id,
        ]);

        $eventB = factory(Event::class)->create();
        $fieldB = factory(Field::class)->create([
            'model_id' => $eventB->id,
            'model_type' => Event::class,
        ]);
        $responseB = factory(Response::class)->create([
            'field_id' => $fieldB->id,
        ]);

        (new DeleteEvent)->execute($eventA);

        $this->assertNull($eventA->fresh());
        $this->assertNull($fieldA->fresh());
        $this->assertNull($responseA->fresh());

        $this->assertNotNull($eventB->fresh());
        $this->assertNotNull($fieldB->fresh());
        $this->assertNotNull($responseB->fresh());
    }
}
